package common

import (
	"bufio"
	"log"
	"net"
	"strings"
	"time"
)

type TCPCommandOutput struct {
	Key   string
	Value interface{}
}

func TCPCommand(target, cmd string, sep string, offset int, timeout time.Duration) (data []TCPCommandOutput, err error) {
	conn, err := net.DialTimeout("tcp", target, timeout)
	if err != nil {
		return
	}
	defer conn.Close()
	if conn.SetDeadline(time.Now().Add(timeout)) != nil {
		log.Println("WARN", "in setting deadline for cmd", target, cmd)
		return
	}
	if _, err = conn.Write([]byte(cmd)); err != nil {
		return
	}
	scanner := bufio.NewScanner(conn)
	for scanner.Scan() {
		line := scanner.Text()
		kv := strings.Split(line, sep)
		if kv[0] == "END" || line == "" || line == "NOT_FOUND" {
			break
		}
		if len(kv) != 2+offset {
			continue
		}
		data = append(data, TCPCommandOutput{Key: kv[offset], Value: kv[1+offset]})
	}
	conn.Close()
	if err = scanner.Err(); err != nil {
		return
	}
	return
}
