package common

import (
	"sync"
	"time"
)

type State struct {
	Services           ServicesMap
	ServicesLastUpdate time.Time
	OnServicesChange   *sync.Cond

	Targets           []*Service
	TargetsLastUpdate time.Time
	OnTargetsChange   *sync.Cond
}

type ServicesMap map[string][]*Service

func NewState() *State {
	return &State{
		Services:           make(ServicesMap),
		ServicesLastUpdate: time.Unix(0, 0),
		OnServicesChange:   sync.NewCond(&sync.Mutex{}),
		Targets:            make([]*Service, 0),
		TargetsLastUpdate:  time.Unix(0, 0),
		OnTargetsChange:    sync.NewCond(&sync.Mutex{}),
	}
}

// Service defines a generic inventory service
type Service struct {
	ID     string `yaml:"id"`
	IP     string `yaml:"ip"`
	Port   int    `yaml:"port"`
	Volume string `yaml:"volume"`
	Type   string
	Conf   map[string]string `yaml:"config"`
}
