package log

import (
	"testing"
	"time"

	"github.com/open-io/oio-exporter/common"
	"github.com/open-io/oio-exporter/testutils"
)

func TestLogModuleLineHandling(t *testing.T) {
	t.Parallel()

	patterns, _, err := parsePatterns("../log_patterns.yml", "")
	if err != nil {
		t.Fatalf("Error parsing patterns: %s", err)
	}

	line := "2020-04-06T07:45:12.359226+00:00 s1-8-gra7-1 OIO,OPENIO,oioswift,0: info  10.0.129.150 10.0.129.150 06/Apr/2020/07/45/12 GET / HTTP/1.0 200 - curl/7.58.0 - 123 456 - tx5cbdd8bb64404d5bbddd0-005e8ade08 - 0.0012 - - 1586159112.357142925 1586159112.358320951 -"
	d := MakeDaemon("", "", time.Second*10)
	rTokenize := patterns["oioswift"]
	numTokens := len(rTokenize.SubexpNames())
	m := common.MakeMetricData()

	// Declare baseLabels beforehand to test for a slice manipulation bug
	// Based on appending several times to the same slice that is not at full capacity
	// Every append after the first one overrides the previous one
	baseLabels := [][]string{
		[]string{"yolo", "swag"},
		[]string{"foo", "bar"},
	}
	if err := d.handleLine(line, rTokenize, numTokens, 1, baseLabels, m); err != nil {
		t.Fatal(err)
	}

	testutils.NewMetricDataTest().
		ForAllMetrics(
			testutils.NewMetricAssertion().
				LabelIs("yolo", "swag").
				LabelIs("foo", "bar").
				LabelIs("method", "GET")).
		ForMetricWithLabels(
			testutils.NewMetricAssertion().Equals(456),
			"log_bandwidth",
			"type", "in").
		ForMetricWithLabels(
			testutils.NewMetricAssertion().Equals(123),
			"log_bandwidth",
			"type", "out").
		ForMetric(
			testutils.NewMetricAssertion().Equals(0.0012),
			"log_response_time").
		ForMetric(
			testutils.NewMetricAssertion().LabelIs("code", "200").Equals(1),
			"log_response_code").
		NoExtraMetrics().
		Check(t, m)
}
